package BibAgenda;

import java.io.*;

class Registro
{
    private static final int TAMANHO_NOME = 30;
    private static final int TAMANHO_TELEFONE = 13;

    public static final int TAMANHO = TAMANHO_NOME + TAMANHO_TELEFONE;

    private String Nome, Telefone;

    private static String replicacao (int Q, char C)
    {
        String R = "";

        for (int i=0; i<Q; i++)
            R = R + C;

        return R;
    }

    public Registro (String N, String T)
    {
        int TN        = Math.min(Registro.TAMANHO_NOME,N.length());
        this.Nome     = N.trim().substring(0,TN);
        int TF        = Math.min(Registro.TAMANHO_TELEFONE,T.length());
        this.Telefone = T.trim().substring(0,TF);
    }

    public Registro (byte B [])
    {
        this (new String (B, 0,                     Registro.TAMANHO_NOME),
              new String (B, Registro.TAMANHO_NOME, Registro.TAMANHO_TELEFONE));
    }

    public String campoNome ()
    {
        return this.Nome;
    }

    public String campoTelefone ()
    {
       return this.Telefone;
    }

    public byte[] seusBytes ()
    {
        String R = this.Nome +
                   Registro.replicacao (
                   Registro.TAMANHO_NOME-this.Nome.length(), ' ') +
                   this.Telefone +
                   Registro.replicacao (
                   Registro.TAMANHO_TELEFONE-this.Telefone.length(), ' ');

        return R.getBytes();
    }
}

public class Agenda
{

    private final int TamQtsCts =  4;

    private RandomAccessFile Arq;

    private int ondeEsta (String N)
    {
        int Inicio = 0, Final = this.quantosContatos() - 1, Onde;

        while (Inicio <= Final)
        {
            Onde = (Inicio + Final) / 2;

            try
            {
                int Comparacao = N.compareTo (this.digaMeNome(Onde));

                if (Comparacao == 0)
                    return Onde;
                else
                    if (Comparacao < 0)
                        Final = Onde - 1;
                    else
                        Inicio = Onde + 1;
            }
            catch (ExcecaoDeAgenda E)
            {}
        }

        return -1;
    }

    private int ondeGuardar (String N)
    {
        int Inicio     = 0,
            Final      = this.quantosContatos() - 1,
            Onde       = 0,
            Comparacao = -1;

        while (Inicio <= Final)
        {
            Onde = (Inicio + Final) / 2;

            try
            {
                Comparacao = N.compareTo (this.digaMeNome(Onde));

                if (Comparacao == 0)
                    return -1;
                else
                    if (Comparacao < 0)
                        Final = Onde - 1;
                    else
                        Inicio = Onde + 1;
            }
            catch (ExcecaoDeAgenda E)
            {}
        }

        return Comparacao < 0? Onde: Onde + 1;
    }

    public Agenda (String NA)
    {
        try
        {
            File DescritorArq = new File (NA);

            if (!DescritorArq.exists ())
            {
                this.Arq = new RandomAccessFile (NA, "rw");
                this.Arq.writeInt (0);
                this.Arq.seek (0L);
            }
            else
                this.Arq = new RandomAccessFile (NA, "rw");
        }
        catch (IOException E)
        {}
    }

    public int quantosContatos ()
    {
        int R = 0;

        try
        {
            long FP = this.Arq.getFilePointer ();

            this.Arq.seek (0L);
            R = this.Arq.readInt ();

            this.Arq.seek (FP);
        }
        catch (IOException E)
        {}

        return R;
    }

    public boolean haRegistroDoContato (String N)
    {
        return this.ondeEsta (N) != -1;
    }

    public void registreOContato (String N, String T) throws ExcecaoDeAgenda
    {
        try
        {
            long FP = this.Arq.getFilePointer ();

            int Posicao = this.ondeGuardar (N);

            if (Posicao == -1)
                ExcecaoDeAgenda.indique (ExcecaoDeAgenda.NOME_JA_REGISTRADO);

            byte B [] = new byte [Registro.TAMANHO];

            for (int I = this.quantosContatos() - 1; I >= Posicao; I--)
            {
                this.Arq.seek  (I*Registro.TAMANHO + TamQtsCts);
                this.Arq.read  (B);
                this.Arq.write (B);
            }

            Registro R = new Registro (N,T);

            this.Arq.seek  (Posicao*Registro.TAMANHO + TamQtsCts);
            this.Arq.write (R.seusBytes());

            this.Arq.seek (0L);
            this.Arq.writeInt (this.quantosContatos()+1);

            this.Arq.seek (FP);
        }
        catch (IOException E)
        {}
    } 

    public String digaMeNome (int P) throws ExcecaoDeAgenda
    {
        Registro R = null;

        try
        {
            long FP = this.Arq.getFilePointer ();

            int QtosContatos = this.quantosContatos ();

            if (QtosContatos == 0)
                ExcecaoDeAgenda.indique (ExcecaoDeAgenda.AGENDA_VAZIA);

            if (P < 0 || P >= QtosContatos)
                ExcecaoDeAgenda.indique (ExcecaoDeAgenda.NOME_FORA_DOS_LIMITES);

            byte B [] = new byte [Registro.TAMANHO];

            this.Arq.seek (P*Registro.TAMANHO + TamQtsCts);
            this.Arq.read (B);

            R = new Registro (B);

            this.Arq.seek (FP);
        }
        catch (IOException E)
        {}

        return R.campoNome();
    }

    public String digaMeTelefone (int P) throws ExcecaoDeAgenda
    {
        Registro R = null;

        try
        {
            long FP = this.Arq.getFilePointer ();

            int QtosContatos = this.quantosContatos ();

            if (QtosContatos == 0)
                ExcecaoDeAgenda.indique (ExcecaoDeAgenda.AGENDA_VAZIA);

            if (P < 0 || P >= QtosContatos)
                ExcecaoDeAgenda.indique (ExcecaoDeAgenda.FONE_FORA_DOS_LIMITES);

            byte B [] = new byte [Registro.TAMANHO];

            this.Arq.seek (P*Registro.TAMANHO + TamQtsCts);
            this.Arq.read (B);

            R = new Registro (B);

            this.Arq.seek (FP);
        }
        catch (IOException E)
        {}

        return R.campoTelefone();
    }

    public String digaMeTelefoneDe (String N) throws ExcecaoDeAgenda
    {
        String R = null;

        if (this.quantosContatos() == 0)
            ExcecaoDeAgenda.indique (ExcecaoDeAgenda.AGENDA_VAZIA);

        int Posicao = this.ondeEsta (N);

        if (Posicao == -1)
            ExcecaoDeAgenda.indique (ExcecaoDeAgenda.NOME_INEXISTENTE);

        try
        {
            R = this.digaMeTelefone (Posicao);;
        }
        catch (ExcecaoDeAgenda E)
        {}

        return R;
    }

    public void descarteContato (String N) throws ExcecaoDeAgenda
    {
        try
        {
            long FP = this.Arq.getFilePointer ();

            if (this.quantosContatos() == 0)
                ExcecaoDeAgenda.indique (ExcecaoDeAgenda.AGENDA_VAZIA);

            int Posicao = this.ondeEsta (N);

            if (Posicao == -1)
                ExcecaoDeAgenda.indique (ExcecaoDeAgenda.NOME_INEXISTENTE);

            int I;

            byte B [] = new byte [Registro.TAMANHO];

            for (I = Posicao; I < this.quantosContatos() - 1; I++)
            {
                this.Arq.seek ((I+1)*Registro.TAMANHO + TamQtsCts);
                this.Arq.read (B);

                this.Arq.seek  (I*Registro.TAMANHO + TamQtsCts);
                this.Arq.write (B);
            }

            this.Arq.setLength (this.Arq.length() - Registro.TAMANHO);

            this.Arq.seek (0L);
            this.Arq.writeInt (this.quantosContatos()-1);

            this.Arq.seek (FP);
        }
        catch (IOException E)
        {}
    }    
}